@@ -1,3 +1,26 @@
+# MIT License
+#
+# Copyright The SCons Foundation
+#
+# Permission is hereby granted, free of charge, to any person obtaining
+# a copy of this software and associated documentation files (the
+# "Software"), to deal in the Software without restriction, including
+# without limitation the rights to use, copy, modify, merge, publish,
+# distribute, sublicense, and/or sell copies of the Software, and to
+# permit persons to whom the Software is furnished to do so, subject to
+# the following conditions:
+#
+# The above copyright notice and this permission notice shall be included
+# in all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
+# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
+# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+
 """
 SCons.Builder
 
@@ -76,42 +99,23 @@ There are the following methods for internal use within this module:
 
 """
 
-#
-# Copyright (c) 2001 - 2019 The SCons Foundation
-#
-# Permission is hereby granted, free of charge, to any person obtaining
-# a copy of this software and associated documentation files (the
-# "Software"), to deal in the Software without restriction, including
-# without limitation the rights to use, copy, modify, merge, publish,
-# distribute, sublicense, and/or sell copies of the Software, and to
-# permit persons to whom the Software is furnished to do so, subject to
-# the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
-# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
-# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
-# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
-# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
-# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
-# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+from __future__ import annotations
 
-__revision__ = "/src/script/src/engine/SCons/Builder.py none 2019-12-16 19:06:27 none"
-
-import collections
+import os
+from collections import UserDict, UserList
+from contextlib import suppress
 
 import SCons.Action
 import SCons.Debug
-from SCons.Debug import logInstanceCreation
-from SCons.Errors import InternalError, UserError
 import SCons.Executor
 import SCons.Memoize
 import SCons.Util
 import SCons.Warnings
+from SCons.Debug import logInstanceCreation
+from SCons.Errors import InternalError, UserError
+from SCons.Executor import Executor
 
-class _Null(object):
+class _Null:
     pass
 
 _null = _Null
@@ -131,14 +135,14 @@ class DictCmdGenerator(SCons.Util.Selector):
     to return the proper action based on the file suffix of
     the source file."""
 
-    def __init__(self, dict=None, source_ext_match=1):
-        SCons.Util.Selector.__init__(self, dict)
+    def __init__(self, mapping=None, source_ext_match: bool=True) -> None:
+        super().__init__(mapping)
         self.source_ext_match = source_ext_match
 
     def src_suffixes(self):
         return list(self.keys())
 
-    def add_action(self, suffix, action):
+    def add_action(self, suffix, action) -> None:
         """Add a suffix-action pair to the mapping.
         """
         self[suffix] = action
@@ -197,7 +201,7 @@ class DictEmitter(SCons.Util.Selector):
             target, source = emitter(target, source, env)
         return (target, source)
 
-class ListEmitter(collections.UserList):
+class ListEmitter(UserList):
     """A callable list of emitters that calls each in sequence,
     returning the result.
     """
@@ -215,7 +219,7 @@ misleading_keywords = {
     'sources'   : 'source',
 }
 
-class OverrideWarner(collections.UserDict):
+class OverrideWarner(UserDict):
     """A class for warning about keyword arguments that we use as
     overrides in a Builder call.
 
@@ -223,14 +227,15 @@ class OverrideWarner(collections.UserDict):
     can actually invoke multiple builders.  This class only emits the
     warnings once, no matter how many Builders are invoked.
     """
-    def __init__(self, dict):
-        collections.UserDict.__init__(self, dict)
+    def __init__(self, mapping) -> None:
+        super().__init__(mapping)
         if SCons.Debug.track_instances: logInstanceCreation(self, 'Builder.OverrideWarner')
         self.already_warned = None
-    def warn(self):
+
+    def warn(self) -> None:
         if self.already_warned:
             return
-        for k in list(self.keys()):
+        for k in self.keys():
             if k in misleading_keywords:
                 alt = misleading_keywords[k]
                 msg = "Did you mean to use `%s' instead of `%s'?" % (alt, k)
@@ -246,7 +251,7 @@ def Builder(**kw):
         kw['action'] = SCons.Action.CommandGeneratorAction(kw['generator'], {})
         del kw['generator']
     elif 'action' in kw:
-        source_ext_match = kw.get('source_ext_match', 1)
+        source_ext_match = kw.get('source_ext_match', True)
         if 'source_ext_match' in kw:
             del kw['source_ext_match']
         if SCons.Util.is_Dict(kw['action']):
@@ -328,14 +333,14 @@ def _node_errors(builder, env, tlist, slist):
         if len(slist) > 1:
             raise UserError("More than one source given for single-source builder: targets=%s sources=%s" % (list(map(str,tlist)), list(map(str,slist))))
 
-class EmitterProxy(object):
+class EmitterProxy:
     """This is a callable class that can act as a
     Builder emitter.  It holds on to a string that
     is a key into an Environment dictionary, and will
     look there at actual build time to see if it holds
     a callable.  If so, we will call that as the actual
     emitter."""
-    def __init__(self, var):
+    def __init__(self, var) -> None:
         self.var = SCons.Util.to_String(var)
 
     def __call__(self, target, source, env):
@@ -354,36 +359,44 @@ class EmitterProxy(object):
 
         return (target, source)
 
-
     def __eq__(self, other):
         return self.var == other.var
 
     def __lt__(self, other):
         return self.var < other.var
 
-class BuilderBase(object):
+    def __le__(self, other):
+        return self.var <= other.var
+
+    def __gt__(self, other):
+        return self.var > other.var
+
+    def __ge__(self, other):
+        return self.var >= other.var
+
+class BuilderBase:
     """Base class for Builders, objects that create output
     nodes (files) from input nodes (files).
     """
 
     def __init__(self,  action = None,
-                        prefix = '',
-                        suffix = '',
-                        src_suffix = '',
+                        prefix: str = '',
+                        suffix: str = '',
+                        src_suffix: str = '',
                         target_factory = None,
                         source_factory = None,
                         target_scanner = None,
                         source_scanner = None,
                         emitter = None,
-                        multi = 0,
+                        multi: bool = False,
                         env = None,
-                        single_source = 0,
+                        single_source: bool = False,
                         name = None,
                         chdir = _null,
-                        is_explicit = 1,
+                        is_explicit: bool = True,
                         src_builder = None,
-                        ensure_suffix = False,
-                        **overrides):
+                        ensure_suffix: bool = False,
+                        **overrides) -> None:
         if SCons.Debug.track_instances: logInstanceCreation(self, 'Builder.BuilderBase')
         self._memo = {}
         self.action = action
@@ -431,12 +444,9 @@ class BuilderBase(object):
             src_builder = [ src_builder ]
         self.src_builder = src_builder
 
-    def __nonzero__(self):
+    def __bool__(self) -> bool:
         raise InternalError("Do not test for the Node.builder attribute directly; use Node.has_builder() instead")
 
-    def __bool__(self):
-        return self.__nonzero__()
-
     def get_name(self, env):
         """Attempts to get the name of the Builder.
 
@@ -466,7 +476,7 @@ class BuilderBase(object):
             suffixes = []
         return match_splitext(path, suffixes)
 
-    def _adjustixes(self, files, pre, suf, ensure_suffix=False):
+    def _adjustixes(self, files, pre, suf, ensure_suffix: bool=False):
         if not files:
             return []
         result = []
@@ -474,6 +484,11 @@ class BuilderBase(object):
             files = [files]
 
         for f in files:
+            # fspath() is to catch PathLike paths. We avoid the simpler
+            # str(f) so as not to "lose" files that are already Nodes:
+            # TypeError: expected str, bytes or os.PathLike object, not File
+            with suppress(TypeError):
+                f = os.fspath(f)
             if SCons.Util.is_String(f):
                 f = SCons.Util.adjustixes(f, pre, suf, ensure_suffix)
             result.append(f)
@@ -504,6 +519,7 @@ class BuilderBase(object):
                 splitext = lambda S: self.splitext(S,env)
                 tlist = [ t_from_s(pre, suf, splitext) ]
         else:
+            # orig_target = target
             target = self._adjustixes(target, pre, suf, self.ensure_suffix)
             tlist = env.arg2nodes(target, target_factory, target=target, source=source)
 
@@ -576,7 +592,7 @@ class BuilderBase(object):
         # build this particular list of targets from this particular list of
         # sources.
 
-        executor = None
+        executor: Executor | None = None
         key = None
 
         if self.multi:
@@ -616,6 +632,10 @@ class BuilderBase(object):
             t.set_executor(executor)
             t.set_explicit(self.is_explicit)
 
+        if env.get("SCONF_NODE"):
+            for node in tlist:
+                node.attributes.conftest_node = 1
+
         return SCons.Node.NodeList(tlist)
 
     def __call__(self, env, target=None, source=None, chdir=_null, **kw):
@@ -647,6 +667,8 @@ class BuilderBase(object):
                 env_kw = kw
         else:
             env_kw = self.overrides
+
+        # TODO if env_kw: then the following line. there's no purpose in calling if no overrides.
         env = env.Override(env_kw)
         return self._execute(env, target, source, OverrideWarner(kw), ekw)
 
@@ -661,7 +683,7 @@ class BuilderBase(object):
             prefix = prefix(env, sources)
         return env.subst(prefix)
 
-    def set_suffix(self, suffix):
+    def set_suffix(self, suffix) -> None:
         if not callable(suffix):
             suffix = self.adjust_suffix(suffix)
         self.suffix = suffix
@@ -672,7 +694,7 @@ class BuilderBase(object):
             suffix = suffix(env, sources)
         return env.subst(suffix)
 
-    def set_src_suffix(self, src_suffix):
+    def set_src_suffix(self, src_suffix) -> None:
         if not src_suffix:
             src_suffix = []
         elif not SCons.Util.is_List(src_suffix):
@@ -686,7 +708,7 @@ class BuilderBase(object):
             return ''
         return ret[0]
 
-    def add_emitter(self, suffix, emitter):
+    def add_emitter(self, suffix, emitter) -> None:
         """Add a suffix-emitter mapping to this Builder.
 
         This assumes that emitter has been initialized with an
@@ -696,7 +718,7 @@ class BuilderBase(object):
         """
         self.emitter[suffix] = emitter
 
-    def add_src_builder(self, builder):
+    def add_src_builder(self, builder) -> None:
         """
         Add a new Builder to the list of src_builders.
 
@@ -863,9 +885,9 @@ class CompositeBuilder(SCons.Util.Proxy):
     to the DictCmdGenerator's add_action() method.
     """
 
-    def __init__(self, builder, cmdgen):
+    def __init__(self, builder, cmdgen) -> None:
         if SCons.Debug.track_instances: logInstanceCreation(self, 'Builder.CompositeBuilder')
-        SCons.Util.Proxy.__init__(self, builder)
+        super().__init__(builder)
 
         # cmdgen should always be an instance of DictCmdGenerator.
         self.cmdgen = cmdgen
@@ -873,11 +895,11 @@ class CompositeBuilder(SCons.Util.Proxy):
 
     __call__ = SCons.Util.Delegate('__call__')
 
-    def add_action(self, suffix, action):
+    def add_action(self, suffix, action) -> None:
         self.cmdgen.add_action(suffix, action)
         self.set_src_suffix(self.cmdgen.src_suffixes())
 
-def is_a_Builder(obj):
+def is_a_Builder(obj) -> bool:
     """"Returns True if the specified obj is one of our Builder classes.
 
     The test is complicated a bit by the fact that CompositeBuilder
