@@ -1,17 +1,6 @@
-"""engine.SCons.Tool.f08
-
-Tool-specific initialization for the generic Posix f08 Fortran compiler.
-
-There normally shouldn't be any need to import this module directly.
-It will usually be imported through the generic SCons.Tool.Tool()
-selection method.
-
-"""
-
-from __future__ import absolute_import
-
+# MIT License
 #
-# Copyright (c) 2001 - 2019 The SCons Foundation
+# Copyright The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -31,28 +20,32 @@ from __future__ import absolute_import
 # LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-#
 
-__revision__ = "/src/script/src/engine/SCons/Tool/f08.py none 2019-12-16 19:06:27 none"
+"""Tool-specific initialization for the generic Posix f08 Fortran compiler.
+
+There normally shouldn't be any need to import this module directly.
+It will usually be imported through the generic SCons.Tool.Tool()
+selection method.
+"""
 
-import SCons.Defaults
-import SCons.Tool
-import SCons.Util
-from . import fortran
 from SCons.Tool.FortranCommon import add_all_to_env, add_f08_to_env
 
 compilers = ['f08']
 
-def generate(env):
+def generate(env) -> None:
     add_all_to_env(env)
     add_f08_to_env(env)
 
     fcomp = env.Detect(compilers) or 'f08'
-    env['F08']  = fcomp
-    env['SHF08']  = fcomp
+    if 'F08' not in env:
+        env['F08']  = fcomp
+    if 'SHF08' not in env:
+        env['SHF08']  = '$F08'
 
-    env['FORTRAN']  = fcomp
-    env['SHFORTRAN']  = fcomp
+    if 'FORTRAN' not in env:
+        env['FORTRAN']  = fcomp
+    if 'SHFORTRAN' not in env:
+        env['SHFORTRAN']  = '$FORTRAN'
 
 
 def exists(env):
